/*
 ******************************* Creazione file *******************************
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago					Data di creazione : 30/03/2000
 * Codice   :
 *
 * Note:
 *
 *
 ******************************* Revisione file *******************************
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


#include <vdw.h>
#include <ntddk.h>
#include "NTPCFOMDriver.h"
#include "NTPCFOMDevice.h"
#include "..\NTPCFOMioctl.h"
#include "NTPCFOMHelper.h"

#include "msglog.h"

#pragma warning(disable:4065) // Allow switch statement with no cases

#pragma code_seg()
////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::DeviceControl
//
//	Routine Description:
//		Handler for IRP_MJ_DEVICE_CONTROL
//
//	Parameters:
//		I - Current IRP
// 
//	Return Value:
//		NTSTATUS - Result code
//
//	Comments:
//		This routine is the first handler for Device Control requests.
//
//		Some function codes may be handled immediately, 
//		while others may be serialized.
//		

NTSTATUS NTPCFOMDevice::DeviceControl(KIrp I) 
{
	NTSTATUS status;

#if DBG
	t << "Entering NTPCFOMDevice::Device Control, " << I;
#endif

	switch (I.IoctlCode())
	{		
		case NTPCFOM_IOCTL_GET_SINGLE_VALUE:
		    status	=	NTPCFOM_IOCTL_GET_SINGLE_VALUE_Handler(I);
			break;
		
		case NTPCFOM_IOCTL_GET_SINGLE_MEDIA_SUM:
			status	=	NTPCFOM_IOCTL_GET_SINGLE_MEDIA_SUM_Handler(I);
			break;

		case NTPCFOM_IOCTL_GET_SINGLE_XSUM_VALUE:
			status	=	NTPCFOM_IOCTL_GET_SINGLE_XSUM_VALUE_Handler(I);
			break;

		case NTPCFOM_IOCTL_GET_MULTIPLE_VALUES:
		    status	=	NTPCFOM_IOCTL_GET_MULTIPLE_VALUES_Handler(I);
			break;

		case NTPCFOM_IOCTL_GET_MULTIPLE_XSUM_VALUES:
		    status	=	NTPCFOM_IOCTL_GET_MULTIPLE_XSUM_VALUES_Handler(I);
			break;

		case NTSTATUS NTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG:
			status	=	NTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG_Handler(I);
			break;
	
		case	NTPCFOM_IOCTL_GET_SINGLE_TRACE_ENABLE:
			status	=	NTPCFOM_IOCTL_GET_SINGLE_TRACE_ENABLE_Handler(I);
			break;

		case	NTPCFOM_IOCTL_SET_SINGLE_TRACE_ENABLE:
			status	=	NTPCFOM_IOCTL_SET_SINGLE_TRACE_ENABLE_Handler(I);
			break;

		case	NTPCFOM_IOCTL_GET_SINGLE_TRACE_INVERT:
			status	=	NTPCFOM_IOCTL_GET_SINGLE_TRACE_INVERT_Handler(I);
			break;
	
		case	NTPCFOM_IOCTL_SET_SINGLE_TRACE_INVERT:
			status	=	NTPCFOM_IOCTL_SET_SINGLE_TRACE_INVERT_Handler(I);
			break;

		case NTPCFOM_IOCTL_GET_SINGLE_TRACE:
			status	=	NTPCFOM_IOCTL_GET_SINGLE_TRACE_Handler(I);
			break;

		case NTPCFOM_IOCTL_GET_MULTIPLE_TRACE_ENABLE:
			status	=	NTPCFOM_IOCTL_GET_MULTIPLE_TRACE_ENABLE_Handler(I);
			break;

		case NTPCFOM_IOCTL_SET_MULTIPLE_TRACE_ENABLE:
			status	=	NTPCFOM_IOCTL_SET_MULTIPLE_TRACE_ENABLE_Handler(I);
			break;

		case NTPCFOM_IOCTL_GET_MULTIPLE_TRACE_INVERT:
			status	=	NTPCFOM_IOCTL_GET_MULTIPLE_TRACE_INVERT_Handler(I);
			break;

		case NTPCFOM_IOCTL_SET_MULTIPLE_TRACE_INVERT:
			status	=	NTPCFOM_IOCTL_SET_MULTIPLE_TRACE_INVERT_Handler(I);
			break;

		case	NTPCFOM_IOCTL_RESET_TRACES_TIMER:
			status	=	NTPCFOM_IOCTL_RESET_TRACES_TIMER_Handler(I);
			break;

		case	NTPCFOM_IOCTL_GET_SAMPLING_DIVISOR_VALUE:
			status	=	NTPCFOM_IOCTL_GET_SAMPLING_DIVISOR_VALUE_Handler(I);
			break;

		case	NTPCFOM_IOCTL_SET_SAMPLING_DIVISOR_VALUE:
			status	=	NTPCFOM_IOCTL_SET_SAMPLING_DIVISOR_VALUE_Handler(I);
			break;

		case	NTPCFOM_IOCTL_RESET:
			status	=	NTPCFOM_IOCTL_RESET_Handler(I);
			break;

		case	NTPCFOM_IOCTL_RESTART:
			status	=	NTPCFOM_IOCTL_RESTART_Handler(I);
			break;

		case NTPCFOM_IOCTL_GET_FIRMWARE_ID:
			status	=	NTPCFOM_IOCTL_GET_FIRMWARE_ID_Handler(I);
			break;

		case NTPCFOM_IOCTL_GET_FIRMWARE_VER:
			status	=	NTPCFOM_IOCTL_GET_FIRMWARE_VER_Handler(I);
			break;
		
		case NTPCFOM_IOCTL_GET_FOM_MODE:
			status	=	NTPCFOM_IOCTL_GET_FOM_MODE_Handler(I);
			break;

		case NTPCFOM_IOCTL_SET_FOM_MODE:
			status	=	NTPCFOM_IOCTL_SET_FOM_MODE_Handler(I);
			break;
			
		default:
			// Unrecognized IOCTL request
			status = STATUS_INVALID_PARAMETER;
			break;
	}

	// If the IRP was queued, or its IOCTL handler deferred processing using some
	// driver specific scheme, the status variable is set to STATUS_PENDING.
	// In this case we simply return that status, and the IRP will be completed
	// later.  Otherwise, complete the IRP using the status returned by the
	// IOCTL handler.
	if (status == STATUS_PENDING)
	{
		return status;
	}
	else
	{
		return I.Complete(status);
	}

//	return QueueIrp(I, LinkTo(CancelQueuedIrp));

	return m_DriverManagedQueue.QueueIrp(I);
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_VALUE_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_SINGLE_VALUE
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_GET_SINGLE_VALUE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_VALUE_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_VALUE_Handler, " << I << EOL;
	t << "sizeof(NTPCFOM_IOCTL_GET_SINGLE_VALUE_BUFFER)" <<	(USHORT)sizeof(NTPCFOM_IOCTL_GET_SINGLE_VALUE_BUFFER)	<< EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_GET_SINGLE_VALUE_BUFFER) || !I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}
	
	USHORT	w_Index	=	*(static_cast<PUSHORT>(I.IoctlBuffer()));
	PNTPCFOM_IOCTL_GET_SINGLE_VALUE_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_GET_SINGLE_VALUE_BUFFER>(I.IoctlBuffer());

	status	=	GetSingleValue(w_Index, &pBuffer->w_Value, &pBuffer->w_Average);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_GET_SINGLE_VALUE_BUFFER);

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_MEDIA_SUM_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_SINGLE_MEDIA_SUM
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_GET_SINGLE_MEDIA_SUM function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_MEDIA_SUM_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_MEDIA_SUM_Handler, " << I << EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(ULONG) || !I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}
	
	USHORT	w_Index	=	*(static_cast<PUSHORT>(I.IoctlBuffer()));
	PULONG	pValue	=	static_cast<PULONG>(I.IoctlBuffer());

	status	=	GetSingleMediaSum(w_Index, pValue);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(ULONG);

	return status;
}


////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_XSUM_VALUE_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_SINGLE_XSUM_VALUE
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_GET_SINGLE_XSUM_VALUE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_XSUM_VALUE_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_XSUM_VALUE_Handler, " << I << EOL;
	t << "sizeof(NTPCFOM_IOCTL_GET_SINGLE_XSUM_VALUES_BUFFER)" <<	(USHORT)sizeof(NTPCFOM_IOCTL_GET_SINGLE_XSUM_VALUES_BUFFER)	<< EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_GET_SINGLE_XSUM_VALUES_BUFFER) || !I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}
	
	USHORT	w_Index	=	*(static_cast<PUSHORT>(I.IoctlBuffer()));
	PNTPCFOM_IOCTL_GET_SINGLE_XSUM_VALUES_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_GET_SINGLE_XSUM_VALUES_BUFFER>(I.IoctlBuffer());

	status	=	GetSingleXSUMValue(w_Index, &pBuffer->w_SampleXSumCt, &pBuffer->ui128_SampleXSum);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_GET_SINGLE_XSUM_VALUES_BUFFER);

	return status;
}


////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_MULTIPLE_VALUES_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_MULTIPLE_VALUES
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_GET_MULTIPLE_VALUE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_MULTIPLE_VALUES_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_GET_MULTIPLE_VALUES_Handler, " << I << EOL;
	t << "sizeof(NTPCFOM_IOCTL_GET_MULTIPLE_VALUES_BUFFER)" <<	(USHORT)sizeof(NTPCFOM_IOCTL_GET_MULTIPLE_VALUES_BUFFER)	<< EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_GET_MULTIPLE_VALUES_BUFFER) || !I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}
	
	
	PNTPCFOM_IOCTL_GET_MULTIPLE_VALUES_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_GET_MULTIPLE_VALUES_BUFFER>(I.IoctlBuffer());

	status	=	GetMultipleValues(&pBuffer->w_Value[0], &pBuffer->w_Average[0]);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}


	I.Information()		=	sizeof(NTPCFOM_IOCTL_GET_MULTIPLE_VALUES_BUFFER);

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_MULTIPLE_XSUM_VALUES_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_MULTIPLE_XSUM_VALUES
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_GET_MULTIPLE_XSUM_VALUE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_MULTIPLE_XSUM_VALUES_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_GET_MULTIPLE_XSUM_VALUES_Handler, " << I << EOL;
	t << "sizeof(NTPCFOM_IOCTL_GET_MULTIPLE_XSUM_VALUES_BUFFER)" <<	(USHORT)sizeof(NTPCFOM_IOCTL_GET_MULTIPLE_XSUM_VALUES_BUFFER)	<< EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_GET_MULTIPLE_XSUM_VALUES_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}
	
	PNTPCFOM_IOCTL_GET_MULTIPLE_XSUM_VALUES_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_GET_MULTIPLE_XSUM_VALUES_BUFFER>(I.IoctlBuffer());

	status	=	GetMultipleXSUMValues(&pBuffer->w_SampleXSumCt, &pBuffer->ui128_SampleXSum[0], &pBuffer->ui128_SampleXSum[1]);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_GET_MULTIPLE_XSUM_VALUES_BUFFER);

	return status;
}


////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_GET_SINGLE_TRACE_CONFIG function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG_Handler, " << I << EOL;
	t << "sizeof(NTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG_BUFFER)" <<	(USHORT)sizeof(NTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG_BUFFER)	<< EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}

	USHORT	w_Index	=	*(static_cast<PUSHORT>(I.IoctlBuffer()));
	PNTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG_BUFFER>(I.IoctlBuffer());

	status	=	GetSingleTraceConfig(w_Index, pBuffer);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG_BUFFER);

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_TRACE_ENABLE_Handler(KIrp I)
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_SINGLE_TRACE_ENABLE
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_GET_SINGLE_TRACE_ENABLE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_TRACE_ENABLE_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_TRACE_ENABLE_Handler, " << I << EOL;
	t << "sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER)" <<	(USHORT)sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER)	<< EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}
	
	PNTPCFOM_IOCTL_SINGLE_STATUS_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_SINGLE_STATUS_BUFFER>(I.IoctlBuffer());

	status	=	GetSingleTraceEnable(pBuffer->w_Index, &pBuffer->w_Value);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER);

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_SET_SINGLE_TRACE_ENABLE_Handler(KIrp I)
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_SET_SINGLE_TRACE_ENABLE
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_SET_SINGLE_TRACE_ENABLE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_SET_SINGLE_TRACE_ENABLE_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_SET_SINGLE_TRACE_ENABLE_Handler, " << I << EOL;
	t << "sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER)" <<	(USHORT)sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER)	<< EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}
	
	PNTPCFOM_IOCTL_SINGLE_STATUS_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_SINGLE_STATUS_BUFFER>(I.IoctlBuffer());

	status	=	SetSingleTraceEnable(pBuffer->w_Index, pBuffer->w_Value);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER);

	return status;
}


////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_TRACE_INVERT_Handler(KIrp I)
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_SINGLE_TRACE_INVERT
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_GET_SINGLE_TRACE_INVERT function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_TRACE_INVERT_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_TRACE_INVERT_Handler, " << I << EOL;
	t << "sizeof(NTPCFOM_IOCTL_GET_SINGLE_STATUS)" <<	(USHORT)sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER)	<< EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}
	
	PNTPCFOM_IOCTL_SINGLE_STATUS_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_SINGLE_STATUS_BUFFER>(I.IoctlBuffer());

	status	=	GetSingleTraceInvert(pBuffer->w_Index, &pBuffer->w_Value);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER);

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_SET_SINGLE_TRACE_INVERT_Handler(KIrp I)
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_SET_SINGLE_TRACE_INVERT
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_SET_SINGLE_TRACE_INVERT function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_SET_SINGLE_TRACE_INVERT_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_SET_SINGLE_TRACE_INVERT_Handler, " << I << EOL;
	t << "sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER)" <<	(USHORT)sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER)	<< EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}
	
	PNTPCFOM_IOCTL_SINGLE_STATUS_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_SINGLE_STATUS_BUFFER>(I.IoctlBuffer());

	status	=	SetSingleTraceInvert(pBuffer->w_Index, pBuffer->w_Value);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_SINGLE_STATUS_BUFFER);

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_TRACE_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_SINGLE_TRACE
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements read the trace dump from board 
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_TRACE_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_GET_SINGLE_TRACE_Handler, " << I << EOL;
	t << "sizeof(NTPCFOM_IOCTL_CHANNEL_TRACE_BUFFER)" <<	(USHORT)sizeof(NTPCFOM_IOCTL_CHANNEL_TRACE_BUFFER)	<< EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_CHANNEL_TRACE_BUFFER) || !I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}

	USHORT	w_Index	=	*(static_cast<PUSHORT>(I.IoctlBuffer()));
	PNTPCFOM_IOCTL_CHANNEL_TRACE_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_CHANNEL_TRACE_BUFFER>(I.IoctlBuffer());

	UCHAR	by_TraceCh	=	(UCHAR)(w_Index&0xff);
	
	if(ValidateTraceID(by_TraceCh)==STATUS_UNSUCCESSFUL)
		return	STATUS_INVALID_PARAMETER;

	if(I.IoctlOutputBufferSize() < (sizeof(NTPCFOM_IOCTL_CHANNEL_TRACE_BUFFER) + (w_size[w_Index] * sizeof (USHORT))) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}

	status	=	GetTrace(by_TraceCh, pBuffer);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	(sizeof(NTPCFOM_IOCTL_CHANNEL_TRACE_BUFFER) + (w_size[w_Index] * sizeof (USHORT)));

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_MULTIPLE_TRACE_ENABLE_Handler(KIrp I)
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_MULTIPLE_TRACE_ENABLE
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_GET_MULTIPLE_TRACE_ENABLE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_MULTIPLE_TRACE_ENABLE_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}

	PNTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER>(I.IoctlBuffer());
	
	status	=	GetMultipleTraceEnable(&pBuffer->w_Value);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER);

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_SET_MULTIPLE_TRACE_ENABLE_Handler(KIrp I)
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_SET_MULTIPLE_TRACE_ENABLE
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_SET_MULTIPLE_TRACE_ENABLE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_SET_MULTIPLE_TRACE_ENABLE_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}

	PNTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER>(I.IoctlBuffer());
	
	status	=	SetMultipleTraceEnable(pBuffer->w_Value);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER);

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_MULTIPLE_TRACE_INVERT_Handler(KIrp I)
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_MULTIPLE_TRACE_INVERT
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_GET_MULTIPLE_TRACE_INVERT function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_MULTIPLE_TRACE_INVERT_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}

	PNTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER>(I.IoctlBuffer());
	
	status	=	GetMultipleTraceInvert(&pBuffer->w_Value);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER);

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_SET_MULTIPLE_TRACE_INVERT_Handler(KIrp I)
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_SET_MULTIPLE_TRACE_INVERT
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_SET_MULTIPLE_TRACE_INVERT function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_SET_MULTIPLE_TRACE_INVERT_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}

	PNTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER>(I.IoctlBuffer());
	
	status	=	SetMultipleTraceInvert(pBuffer->w_Value);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_MULTIPLE_STATUS_BUFFER);

	return status;
}


////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_RESET_TRACES_TIMER_Handler(KIrp I)
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_RESET_TRACES_TIMER
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the hardware PC_PCCFOM_TRACE_TIMER_RESET function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_RESET_TRACES_TIMER_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_RESET_TRACES_TIMER_Handler, " << I << EOL;
#endif

	status	=	ResetTracesTimer();

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_SAMPLING_DIVISOR_VALUE_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_SAMPLING_DIVISOR_VALUE
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_GET_SAMPLING_DIVISOR_VALUE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_SAMPLING_DIVISOR_VALUE_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "NTPCFOMDevice::NTPCFOM_IOCTL_GET_CH23_DIVISOR_VALUE_Handler, " << I << EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_SAMPLE_DIVISOR_VALUE_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}

	PNTPCFOM_IOCTL_SAMPLE_DIVISOR_VALUE_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_SAMPLE_DIVISOR_VALUE_BUFFER>(I.IoctlBuffer());
	
	status	=	GetSamplingDivisorValue(pBuffer->w_Index, &pBuffer->w_SampleDivisorValue);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_SAMPLE_DIVISOR_VALUE_BUFFER);

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_SET_SAMPLING_DIVISOR_VALUE_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_SET_SAMPLING_DIVISOR_VALUE
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_SET_SAMPLING_DIVISOR_VALUE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_SET_SAMPLING_DIVISOR_VALUE_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "NTPCFOMDevice::NTPCFOM_IOCTL_SET_SAMPLING_DIVISOR_VALUE_Handler, " << I << EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_SAMPLE_DIVISOR_VALUE_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}

	PNTPCFOM_IOCTL_SAMPLE_DIVISOR_VALUE_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_SAMPLE_DIVISOR_VALUE_BUFFER>(I.IoctlBuffer());
	
	status	=	SetSamplingDivisorValue(pBuffer->w_Index, pBuffer->w_SampleDivisorValue);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_SAMPLE_DIVISOR_VALUE_BUFFER);

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_RESET_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_RESET
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements hardware Reset function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_RESET_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;
	
	status	=	Reset();

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_RESTART_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_RESTART
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements hardware Restart function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_RESTART_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;
	
	status	=	Restart();

	return status;
}

////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_FIRMWARE_VER_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_FIRMWARE_VER
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_CMD_GET_FIRMWARE_VERSION function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_FIRMWARE_VER_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOM_IOCTL_GET_FIRMWARE_VER_Handler, " << I << EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_FIRMWARE_ID_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}

	PNTPCFOM_IOCTL_FIRMWARE_VER_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_FIRMWARE_VER_BUFFER>(I.IoctlBuffer());
	
	status	=	GetFirmwareVersion(&pBuffer->w_MajorVersion, &pBuffer->w_MinorVersion);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(PNTPCFOM_IOCTL_FIRMWARE_VER_BUFFER);

	return status;
}


////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_FIRMWARE_ID_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_FIRMWARE_ID
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_IOCTL_CMD_TRACEW_VALUE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_FIRMWARE_ID_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_GETFIRMWAREID_Handler, " << I << EOL;
//	t << "sizeof(NTPCFOM_IOCTL_GETCONFIG_BUFFER)" <<	(USHORT)sizeof(NTPCFOM_IOCTL_GETCONFIG_BUFFER)	<< EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(NTPCFOM_IOCTL_FIRMWARE_ID_BUFFER) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}

	PNTPCFOM_IOCTL_FIRMWARE_ID_BUFFER	pBuffer	=	static_cast<PNTPCFOM_IOCTL_FIRMWARE_ID_BUFFER>(I.IoctlBuffer());
	
	status	=	GetFirmwareID(pBuffer->by_Buffer);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(NTPCFOM_IOCTL_FIRMWARE_ID_BUFFER);

	return status;
}


////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_GET_FOM_MODE_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_FIRMWARE_ID
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_IOCTL_CMD_TRACEW_VALUE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_GET_FOM_MODE_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_GET_FOM_MODE_Handler, " << I << EOL;
#endif

	if(I.IoctlOutputBufferSize() < sizeof(USHORT) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}
	
	PUSHORT	pValue	=	static_cast<PUSHORT>(I.IoctlBuffer());

	status	=	GetFOMMode(pValue);

	if(!NT_SUCCESS(status))
	{
		I.Information()	=	0;
		return	STATUS_UNEXPECTED_IO_ERROR;
	}

	I.Information()		=	sizeof(USHORT);

	return status;
}


////////////////////////////////////////////////////////////////////////
//  NTPCFOMDevice::NTPCFOM_IOCTL_SET_FOM_MODE_Handler
//
//	Routine Description:
//		Handler for IO Control Code NTPCFOM_IOCTL_GET_FIRMWARE_ID
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the PCCFOM_IOCTL_CMD_TRACEW_VALUE function.
//		This routine runs at passive level.
//

NTSTATUS NTPCFOMDevice::NTPCFOM_IOCTL_SET_FOM_MODE_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

#if DBG
	t << "Entering NTPCFOMDevice::NTPCFOM_IOCTL_SET_FOM_MODE_Handler, " << I << EOL;
#endif

	if(I.IoctlInputBufferSize() < sizeof(USHORT) || ! I.IoctlBuffer())
	{
		I.Information()	=	0;
		return	STATUS_INVALID_PARAMETER;
	}
	
	PUSHORT	pValue	=	static_cast<PUSHORT>(I.IoctlBuffer());

	status	=	SetFOMMode(*pValue);

	I.Information()		=	0;
	
	return status;
}